@@ -38,11 +38,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/dir.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <termios.h>
 #include <unistd.h>
+#include <time.h>
 
 #include "c.h"
 #include "fileutils.h"
@@ -72,6 +72,12 @@ static int statMode = VMSTAT;
 /* "-a" means "show active/inactive" */
 static int a_option;
 
+/* "-w" means "wide output" */
+static int w_option;
+
+/* "-t" means "show timestamp" */
+static int t_option;
+
 static unsigned sleep_time = 1;
 static int infinite_updates = 0;
 static unsigned long num_updates;
@@ -87,15 +93,17 @@ static void __attribute__ ((__noreturn__))
 	      _(" %s [options] [delay [count]]\n"),
 		program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -a, --active           active/inactive memory\n"
-		" -f, --forks            number of forks since boot\n"
-		" -m, --slabs            slabinfo\n"
-		" -n, --one-header       do not redisplay header\n"
-		" -s, --stats            event counter statistics\n"
-		" -d, --disk             disk statistics\n"
-		" -D, --disk-sum         summarize disk statistics\n"
-		" -p, --partition <dev>  partition specific statistics\n"
-		" -S, --unit <char>      define display unit\n"), out);
+	fputs(_(" -a, --active           active/inactive memory\n"), out);
+	fputs(_(" -f, --forks            number of forks since boot\n"), out);
+	fputs(_(" -m, --slabs            slabinfo\n"), out);
+	fputs(_(" -n, --one-header       do not redisplay header\n"), out);
+	fputs(_(" -s, --stats            event counter statistics\n"), out);
+	fputs(_(" -d, --disk             disk statistics\n"), out);
+	fputs(_(" -D, --disk-sum         summarize disk statistics\n"), out);
+	fputs(_(" -p, --partition <dev>  partition specific statistics\n"), out);
+	fputs(_(" -S, --unit <char>      define display unit\n"), out);
+	fputs(_(" -w, --wide             wide output\n"), out);
+	fputs(_(" -t, --timestamp        show timestamp\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
 	fputs(USAGE_VERSION, out);
@@ -176,12 +184,35 @@ static int format_1000(unsigned long long val64, char *restrict dst)
 
 static void new_header(void)
 {
+	struct tm *tm_ptr;
+	time_t the_time;
+	char timebuf[32];
+
 	/* Translation Hint: Translating folloging header & fields
 	 * that follow (marked with max x chars) might not work,
 	 * unless manual page is translated as well.  */
-	printf(_("procs -----------memory---------- ---swap-- -----io---- -system-- ----cpu----\n"));
-	printf
-	    ("%2s %2s %6s %6s %6s %6s %4s %4s %5s %5s %4s %4s %2s %2s %2s %2s\n",
+	const char *header =
+	    _("procs -----------memory---------- ---swap-- -----io---- -system-- ------cpu-----");
+	const char *wide_header =
+	    _("procs -----------------------memory---------------------- ---swap-- -----io---- -system-- --------cpu--------");
+	const char *timestamp_header = _(" -----timestamp-----");
+
+	const char format[] =
+	    "%2s %2s %6s %6s %6s %6s %4s %4s %5s %5s %4s %4s %2s %2s %2s %2s %2s";
+	const char wide_format[] =
+	    "%2s %2s %12s %12s %12s %12s %4s %4s %5s %5s %4s %4s %3s %3s %3s %3s %3s";
+
+
+	printf("%s", w_option ? wide_header : header);
+
+	if (t_option) {
+		printf("%s", timestamp_header);
+	}
+
+	printf("\n");
+
+	printf(
+	    w_option ? wide_format : format,
 	    /* Translation Hint: max 2 chars */
 	     _("r"),
 	    /* Translation Hint: max 2 chars */
@@ -217,7 +248,22 @@ static void new_header(void)
 	    /* Translation Hint: max 2 chars */
 	     _("id"),
 	    /* Translation Hint: max 2 chars */
-	     _("wa"));
+	     _("wa"),
+	    /* Translation Hint: max 2 chars */
+	     _("st"));
+
+	if (t_option) {
+		(void) time( &the_time );
+		tm_ptr = localtime( &the_time );
+		if (strftime(timebuf, sizeof(timebuf), "%Z", tm_ptr)) {
+			timebuf[strlen(timestamp_header) - 1] = '\0';
+		} else {
+			timebuf[0] = '\0';
+		}
+		printf(" %*s", (int)(strlen(timestamp_header) - 1), timebuf);
+	}
+
+	printf("\n");
 }
 
 static unsigned long unitConvert(unsigned long size)
@@ -230,7 +276,10 @@ static unsigned long unitConvert(unsigned long size)
 static void new_format(void)
 {
 	const char format[] =
-	    "%2u %2u %6lu %6lu %6lu %6lu %4u %4u %5u %5u %4u %4u %2u %2u %2u %2u\n";
+	    "%2u %2u %6lu %6lu %6lu %6lu %4u %4u %5u %5u %4u %4u %2u %2u %2u %2u %2u";
+	const char wide_format[] =
+	    "%2u %2u %12lu %12lu %12lu %12lu %4u %4u %5u %5u %4u %4u %3u %3u %3u %3u %3u";
+
 	unsigned int tog = 0;	/* toggle switch for cleaner code */
 	unsigned int i;
 	unsigned int hz = Hertz;
@@ -243,6 +292,9 @@ static void new_format(void)
 	unsigned int sleep_half;
 	unsigned long kb_per_page = sysconf(_SC_PAGESIZE) / 1024ul;
 	int debt = 0;		/* handle idle ticks running backwards */
+	struct tm *tm_ptr;
+	time_t the_time;
+	char timebuf[32];
 
 	sleep_half = (sleep_time / 2);
 	new_header();
@@ -252,6 +304,12 @@ static void new_format(void)
 		cpu_zzz, pgpgin, pgpgout, pswpin, pswpout, intr, ctxt, &running,
 		&blocked, &dummy_1, &dummy_2);
 
+	if (t_option) {
+		(void) time( &the_time );
+		tm_ptr = localtime( &the_time );
+		strftime(timebuf, sizeof(timebuf), "%Y-%m-%d %H:%M:%S", tm_ptr);
+	}
+
 	duse = *cpu_use + *cpu_nic;
 	dsys = *cpu_sys + *cpu_xxx + *cpu_yyy;
 	didl = *cpu_idl;
@@ -260,7 +318,7 @@ static void new_format(void)
 	Div = duse + dsys + didl + diow + dstl;
 	if (!Div) Div = 1, didl = 1;
 	divo2 = Div / 2UL;
-	printf(format,
+	printf(w_option ? wide_format : format,
 	       running, blocked,
 	       unitConvert(kb_swap_used), unitConvert(kb_main_free),
 	       unitConvert(a_option?kb_inactive:kb_main_buffers),
@@ -274,10 +332,16 @@ static void new_format(void)
 	       (unsigned)( (100*duse			+ divo2) / Div ),
 	       (unsigned)( (100*dsys			+ divo2) / Div ),
 	       (unsigned)( (100*didl			+ divo2) / Div ),
-	       (unsigned)( (100*diow			+ divo2) / Div )/*,
-	       (unsigned)( (100*dstl			+ divo2) / Div ) */
+	       (unsigned)( (100*diow			+ divo2) / Div ),
+	       (unsigned)( (100*dstl			+ divo2) / Div )
 	);
 
+	if (t_option) {
+		printf(" %s", timebuf);
+	}
+
+	printf("\n");
+
 	/* main loop */
 	for (i = 1; infinite_updates || i < num_updates; i++) {
 		sleep(sleep_time);
@@ -293,6 +357,12 @@ static void new_format(void)
 			pgpgout + tog, pswpin + tog, pswpout + tog, intr + tog,
 			ctxt + tog, &running, &blocked, &dummy_1, &dummy_2);
 
+		if (t_option) {
+			(void) time( &the_time );
+			tm_ptr = localtime( &the_time );
+			strftime(timebuf, sizeof(timebuf), "%Y-%m-%d %H:%M:%S", tm_ptr);
+		}
+
 		duse =
 		    cpu_use[tog] - cpu_use[!tog] + cpu_nic[tog] - cpu_nic[!tog];
 		dsys =
@@ -315,7 +385,7 @@ static void new_format(void)
 		Div = duse + dsys + didl + diow + dstl;
 		if (!Div) Div = 1, didl = 1;
 		divo2 = Div / 2UL;
-		printf(format,
+		printf(w_option ? wide_format : format,
 		       running,
 		       blocked,
 		       unitConvert(kb_swap_used),unitConvert(kb_main_free),
@@ -340,31 +410,33 @@ static void new_format(void)
 		       /* id */
 		       (unsigned)( (100*didl+divo2)/Div ),
 		       /* wa */
-		       (unsigned)( (100*diow+divo2)/Div )/*,
-		       / * st  * /
-		       (unsigned)( (100*dstl+divo2)/Div ) */
+		       (unsigned)( (100*diow+divo2)/Div ),
+		       /* st */
+		       (unsigned)( (100*dstl+divo2)/Div )
 		);
+
+		if (t_option) {
+			printf(" %s", timebuf);
+		}
+
+		printf("\n");
 	}
 }
 
 static void diskpartition_header(const char *partition_name)
 {
-	printf("%-10s %10s %10s %10s %10s\n",
+	printf("%-10s %10s  %16s  %10s  %16s\n",
 	       partition_name,
        /* Translation Hint: Translating folloging disk partition
 	* header fields that follow (marked with max x chars) might
 	* not work, unless manual page is translated as well. */
-	       /* Translation Hint: max 10 chars. The word is
-	        * expected to be centralized, use spaces at the end
-	        * to do that. */
-	       _("reads  "),
-	       /* Translation Hint: max 10 chars */
+	       /* Translation Hint: max 10 chars. */
+	       _("reads"),
+	       /* Translation Hint: max 16 chars. */
 	       _("read sectors"),
-	       /* Translation Hint: max 10 chars. The word is
-	        * expected to be centralized, use spaces at the end
-	        * to do that. */
-	       _("writes   "),
-	       /* Translation Hint: max 10 chars */
+	       /* Translation Hint: max 10 chars. */
+	       _("writes"),
+	       /* Translation Hint: max 16 chars */
 	       _("requested writes"));
 }
 
@@ -374,7 +446,7 @@ static int diskpartition_format(const char *partition_name)
 	struct disk_stat *disks;
 	struct partition_stat *partitions, *current_partition = NULL;
 	unsigned long ndisks, j, k, npartitions;
-	const char format[] = "%20u %10llu %10u %10llu\n";
+	const char format[] = "%21u  %16llu  %10u  %16llu\n";
 
 	fDiskstat = fopen("/proc/diskstats", "rb");
 	if (!fDiskstat)
@@ -433,11 +505,33 @@ static int diskpartition_format(const char *partition_name)
 
 static void diskheader(void)
 {
+	struct tm *tm_ptr;
+	time_t the_time;
+	char timebuf[32];
+
 	/* Translation Hint: Translating folloging header & fields
 	 * that follow (marked with max x chars) might not work,
 	 * unless manual page is translated as well.  */
-	printf(_("disk- ------------reads------------ ------------writes----------- -----IO------\n"));
-	printf("%5s %6s %6s %7s %7s %6s %6s %7s %7s %6s %6s\n",
+	const char *header =
+	    _("disk- ------------reads------------ ------------writes----------- -----IO------");
+	const char *wide_header =
+	    _("disk- -------------------reads------------------- -------------------writes------------------ ------IO-------");
+	const char *timestamp_header = _(" -----timestamp-----");
+
+	const char format[] =
+	    "%5s %6s %6s %7s %7s %6s %6s %7s %7s %6s %6s";
+	const char wide_format[] =
+	    "%5s %9s %9s %11s %11s %9s %9s %11s %11s %7s %7s";
+
+	printf("%s", w_option ? wide_header : header);
+
+	if (t_option) {
+		printf("%s", timestamp_header);
+	}
+
+	printf("\n");
+
+	printf(w_option ? wide_format : format,
 	       " ",
 	       /* Translation Hint: max 6 chars */
 	       _("total"),
@@ -459,25 +553,53 @@ static void diskheader(void)
 	       _("cur"),
 	       /* Translation Hint: max 6 chars */
 	       _("sec"));
+
+	if (t_option) {
+		(void) time( &the_time );
+		tm_ptr = localtime( &the_time );
+		if (strftime(timebuf, sizeof(timebuf), "%Z", tm_ptr)) {
+			timebuf[strlen(timestamp_header) - 1] = '\0';
+		} else {
+			timebuf[0] = '\0';
+		}
+		printf(" %*s", (int)(strlen(timestamp_header) - 1), timebuf);
+	}
+
+	printf("\n");
 }
 
 static void diskformat(void)
 {
+	const char format[] =
+	    "%-5s %6u %6u %7llu %7u %6u %6u %7llu %7u %6u %6u";
+	const char wide_format[] =
+	    "%-5s %9u %9u %11llu %11u %9u %9u %11llu %11u %7u %7u";
+
 	FILE *fDiskstat;
 	struct disk_stat *disks;
 	struct partition_stat *partitions;
 	unsigned long ndisks, i, j, k;
-	const char format[] = "%-5s %6u %6u %7llu %7u %6u %6u %7llu %7u %6u %6u\n";
+	struct tm *tm_ptr;
+	time_t the_time;
+	char timebuf[32];
+
 
 	if ((fDiskstat = fopen("/proc/diskstats", "rb"))) {
 		fclose(fDiskstat);
 		ndisks = getdiskstat(&disks, &partitions);
+
+		if (t_option) {
+			(void) time( &the_time );
+			tm_ptr = localtime( &the_time );
+			strftime(timebuf, sizeof(timebuf), "%Y-%m-%d %H:%M:%S", tm_ptr);
+		}
+
 		if (!moreheaders)
 			diskheader();
 		for (k = 0; k < ndisks; k++) {
 			if (moreheaders && ((k % height) == 0))
 				diskheader();
-			printf(format,
+			printf(w_option ? wide_format : format,
 			       disks[k].disk_name,
 			       disks[k].reads,
 			       disks[k].merged_reads,
@@ -490,31 +612,51 @@ static void diskformat(void)
 			       disks[k].inprogress_IO ? disks[k].inprogress_IO / 1000 : 0,
 			       disks[k].milli_spent_IO ? disks[k].
 			       milli_spent_IO / 1000 : 0);
+
+			if (t_option) {
+				printf(" %s", timebuf);
+			}
+
+			printf("\n");
 			fflush(stdout);
 		}
 		free(disks);
 		free(partitions);
+
 		for (j = 1; infinite_updates || j < num_updates; j++) {
 			sleep(sleep_time);
 			ndisks = getdiskstat(&disks, &partitions);
-		for (i = 0; i < ndisks; i++, k++) {
-			if (moreheaders && ((k % height) == 0))
-				diskheader();
-			printf(format,
-			       disks[i].disk_name,
-			       disks[i].reads,
-			       disks[i].merged_reads,
-			       disks[i].reads_sectors,
-			       disks[i].milli_reading,
-			       disks[i].writes,
-			       disks[i].merged_writes,
-			       disks[i].written_sectors,
-			       disks[i].milli_writing,
-			       disks[i].inprogress_IO ? disks[i].inprogress_IO / 1000 : 0,
-			       disks[i].milli_spent_IO ? disks[i].
-			       milli_spent_IO / 1000 : 0);
-			fflush(stdout);
-		}
+
+			if (t_option) {
+				(void) time( &the_time );
+				tm_ptr = localtime( &the_time );
+				strftime(timebuf, sizeof(timebuf), "%Y-%m-%d %H:%M:%S", tm_ptr);
+			}
+
+			for (i = 0; i < ndisks; i++, k++) {
+				if (moreheaders && ((k % height) == 0))
+					diskheader();
+				printf(w_option ? wide_format : format,
+				       disks[i].disk_name,
+				       disks[i].reads,
+				       disks[i].merged_reads,
+				       disks[i].reads_sectors,
+				       disks[i].milli_reading,
+				       disks[i].writes,
+				       disks[i].merged_writes,
+				       disks[i].written_sectors,
+				       disks[i].milli_writing,
+				       disks[i].inprogress_IO ? disks[i].inprogress_IO / 1000 : 0,
+				       disks[i].milli_spent_IO ? disks[i].
+				       milli_spent_IO / 1000 : 0);
+
+				if (t_option) {
+					printf(" %s", timebuf);
+				}
+
+				printf("\n");
+				fflush(stdout);
+			}
 			free(disks);
 			free(partitions);
 		}
@@ -719,6 +861,8 @@ int main(int argc, char *argv[])
 		{"disk-sum", no_argument, NULL, 'D'},
 		{"partition", required_argument, NULL, 'p'},
 		{"unit", required_argument, NULL, 'S'},
+		{"wide", no_argument, NULL, 'w'},
+		{"timestamp", no_argument, NULL, 't'},
 		{"help", no_argument, NULL, 'h'},
 		{"version", no_argument, NULL, 'V'},
 		{NULL, 0, NULL, 0}
@@ -733,7 +877,7 @@ int main(int argc, char *argv[])
 	atexit(close_stdout);
 
 	while ((c =
-		getopt_long(argc, argv, "afmnsdDp:S:hV", longopts,
+		getopt_long(argc, argv, "afmnsdDp:S:wthV", longopts,
 			    NULL)) != EOF)
 		switch (c) {
 		case 'V':
@@ -796,6 +940,12 @@ int main(int argc, char *argv[])
 		case 's':
 			statMode |= VMSUMSTAT;
 			break;
+		case 'w':
+			w_option = 1;
+			break;
+		case 't':
+			t_option = 1;
+			break;
 		default:
 			/* no other aguments defined yet. */
 			usage(stderr);
